@@ -8,7 +8,7 @@ import           Data.ByteString (ByteString)
 import           Blaze.ByteString.Builder
 import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Builder as B
-import           Data.Char (isAscii, isLatin1)
+import           Data.Char (chr, isAscii, isLatin1)
 import qualified Data.HashMap.Strict as M
 import qualified Data.HashSet as S
 import qualified Data.Map as Map
@@ -21,6 +21,8 @@ import qualified Data.Text.Encoding.Error as TE
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Encoding as TL
 
+import           Text.Parsec.Text (Parser)
+
 import           Text.XmlHtml.HTML.Meta (reversePredefinedRefs,
                                          explicitAttributes)
 
@@ -246,7 +248,7 @@ encoder :: Encoding -> Text -> ByteString
 encoder UTF8       = T.encodeUtf8
 encoder UTF16BE    = T.encodeUtf16BE
 encoder UTF16LE    = T.encodeUtf16LE
-encoder ISO_8859_1 = encodeIso_8859_1
+encoder ISO_8859_1 = encodeAscii
 
 
 ------------------------------------------------------------------------------
@@ -254,9 +256,9 @@ encoder ISO_8859_1 = encodeIso_8859_1
 -- UTF-8 characters found in the input and present in the
 -- 'Text.XmlHtml.Meta.references' map are mapped to their escape sequences,
 -- and any other UTF-8 characters are replaced with ascii "?"
-encodeIso_8859_1 :: Text -> ByteString
-encodeIso_8859_1 t = T.encodeUtf8 . T.concat . map toAsciiChunk $
-                     T.groupBy asciiSplits t
+encodeAscii :: Text -> ByteString
+encodeAscii t = T.encodeUtf8 . T.concat . map toAsciiChunk $
+                T.groupBy asciiSplits t
   where
 
     -- Identify long strings of all-acceptable or all-unacceptable characters
@@ -306,3 +308,11 @@ bmap f   = B.byteString
                . TL.toStrict
                . TL.decodeUtf8
                . B.toLazyByteString
+
+------------------------------------------------------------------------------
+-- Parse a character from its character code, failing if invalid.
+safeChr :: Int -> Parser Char
+safeChr c =
+    if c > 1114111
+      then fail ("Invalid character code: " ++ show c)
+      else return (chr c)
